@@ -90,14 +90,14 @@ def get_component(component_type: str) -> typing.Optional[typing.Any]:
 
     def attrgetter(attrname: str, default: typing.Optional[int] = None) -> typing.Callable[[typing.Any], int]:
         def inside(obj: typing.Any) -> int:
-            return int(getattr(obj, attrname, default))
+            return int(typing.cast(int, getattr(obj, attrname, default)))
 
         return inside
 
     sorted_components = sorted(components, key=attrgetter("priority", 50), reverse=True)
 
     if len(sorted_components) > 0:
-        return tuple(components)[0]
+        return tuple(sorted_components)[0]
 
     return None
 
